/* Regression webinar Stata 2024/06/04 */

**# Linear regression
clear all
webuse lbw
list bwt age race smoke in 1/20, noobs sep(0)

dtable bwt age i.race i.smoke

regress bwt age i.race i.smoke
estimates store linear

set showbaselevels on, perm

regress, noheader

regress, notable

preserve
duplicates drop age, force
quietly regress bwt age
local bage = _b[age]
local cons = _b[_cons]

forvalues i = 1/`=_N'{
local y2 = bwt[`i']
local x2 = age[`i']
local x1 = age[`i']
local y1 = age[`i']*`bage' + `cons'
local guts1 "(pcarrowi `y2' `x2' `y1' `x1'" 
local guts2 ", color(stc2) lw(thin) lc(%60) mc(stc2) mlc(%60))"
local guts "`guts' `guts1'`guts2'"
}

graph twoway (lfit bwt age) (scatter bwt age) ///
			 `guts', legend(order(1 "regression line" 2 "actual value")) ///
			 ytitle("Birthweight (grams)") xtitle("Age of mother")
restore				 

regress bwt age i.race i.smoke, base vce(robust)
estimates store robust

set linesize 79
regress bwt age i.race i.smoke, vce(bootstrap, reps(100) seed(123))
estimates store boot
set linesize 95

etable, estimates(linear robust boot)

regress bwt c.age##c.age i.race##i.smoke, base

test 2.race == 3.race

testparm race#smoke

margins, at(smoke = 0)
margins, at(smoke = 0 race = 1)

margins, at(age=(14(5)45))

marginsplot

nlcom -_b[age]/(2*_b[age#age])

local x1 = -_b[age]/(2*_b[age#age])
preserve
margins, at(age = `x1') post
local y1 = _b[_cons]
restore
local x2 = `x1' + 2
local y2 = `y1' - 100
regress bwt c.age##c.age race##smoke, base

quietly margins, at(age=(14(1)45))

marginsplot, noci   ///
   addplot(pcarrowi `y2' `x2' `y1' `x1' (3) "min of bwt at age = 25" || ///
           pci `y1' 20 `y1' 30) ///
   recast(line) legend(off) ///
   title("Quadratic relationship between bwt and age") ///
   ytitle("Prediction of birthweight")


margins, dydx(age)
quietly margins, dydx(age) atmeans

margins, dydx(age) at(age=(14(5)45))

marginsplot, yline(0) xline(25.3)

regress bwt c.age##c.age i.race##i.smoke, base
est store regress

local guts
forvalues i = 15(3)45{
local x1 = `i'

est restore regress
margins, at(age = `x1') post
local y1 = _b[_cons]

est restore regress
margins, dydx(age) at(age = `i') post
local b = _b[age]

local cons = `y1' - `b'*`x1'
macro list

local guts "`guts' (function y = `cons' + `b'*x, range(`=`i'-5' `=`i'+3') lcolor(red) lwidth(vthin))"

}

regress bwt c.age##c.age race##smoke, base

quietly margins, at(age=(14(1)45))
marginsplot, noci   ///
   addplot(`guts') ///
   recast(line) legend(off) ///  
   title("Quadratic relationship between bwt and age") ///
   ytitle("Prediction of birthweight") plot1opts(lwidth(thick))
   
  
margins race#smoke

marginsplot, recast(scatter) name(m1, replace)
marginsplot, recastci(rarea) ciopts(fcolor(%20) acolor(%20)) name(m2, replace)
graph combine m1 m2, iscale(0.7)

**# Nonlinear models

list bwt low lwt age race smoke in 120/140, noobs sep(0)

probit low age i.race##c.lwt i.smoke, base

margins, dydx(age)
margins, dydx(age) atmeans

margins smoke, at(age = 25 race = 1)
margins r.smoke, at(age = 25 race = 1)

quietly margins r.race if race == 1 | race == 3, at(lwt=(80(10)250))
marginsplot, yline(0) title("White vs Other")


margins, at(age=generate(age)) at(age=generate(age+4))
margins, at(age=generate(age)) at(age=generate(age+4)) contrast(atcontrast(r))

quietly margins race, at(lwt=(80(10)250)) 
marginsplot, noci


logit low age i.race##c.lwt i.smoke, base
quietly logistic low age i.race##c.lwt i.smoke, base

twoway function normal = normal(x), range(-10 10) || ///
   function logistic = logistic(x), range(-10 10) || ///
   function difference = abs(normal(x) - logistic(x)), lpattern(dash) ///
   lcolor(black) range(-10 10) title("Normal and Logistic CDFs") ytitle("F(x)")


webuse dollhill3, clear
list deaths smoke agecat pyears, noobs sep(0)

poisson deaths smokes i.agecat, exposure(pyears) vce(robust)

margins, predict(pr(5)) 

margins agecat, predict(n)

**# Instrumental estimation

webuse nlswork, clear
describe ln_wage age race tenure union msp occ_code

ivregress 2sls ln_wage c.age##c.age i.race (tenure = union msp occ_code)

ivregress 2sls ln_wage c.age##c.age i.race (tenure = union msp occ_code), first

estat firststage

estat weakrobust

estat endogenous
estat overid



